#!/usr/bin/env python3
r"""
Used by CI to bootstrap a new cluster and run tests.

Env vars needed:
- COSMOS_API_TOKEN

Env vars optional:
- NO_TERMINATE_DROPLETS: list of droplet ids that should not be
  removed when deleting droplets.

Install requirements:

- Alpine using `requirements.txt`:

    apk --no-cache add python3-dev build-base libffi-dev linux-headers \
      openssl-dev openssh-client
    pip3 install -r requirements.txt

- Apline using packages (much faster):

    apk --no-cache add ansible musl-dev linux-headers gcc py3-psutil \
      openssh-client
    pip3 install requests tabulate testinfra


- Debian (using deb packages):
    apt-get install -y --no-install-recommends ansible gcc libc6-dev \
      python3-pip python3-setuptools python3-wheel \
      python3-psutil
    pip3 install requests tabulate testinfra
"""

import argparse
import logging
import os
import sys
from behave.__main__ import main as behave_main
from openappstack import name, cluster, cosmos, ansible



def main():  # pylint: disable=too-many-statements,too-many-branches,too-many-locals
    """Do everything."""
    # Parse command line arguments
    parser = argparse.ArgumentParser(
        prog=name,
        description='Run bootstrap script '
        'to deploy Openappstack to a given node.')

    parser.add_argument(
        'cluster_name',
        metavar='CLUSTER_NAME',
        type=str,
        help="Name of the cluster you want to use openappstack with")

    subparsers = parser.add_subparsers(help="Available subcommands")

    create_parser = subparsers.add_parser('create', help='Creates a new cluster')
    create_parser.set_defaults(func=create)

    create_parser.add_argument(
        'domain',
        metavar="DOMAIN_NAME",
        help='Domain name to run OpenAppStack under')

    group = create_parser.add_mutually_exclusive_group(required=True)

    group.add_argument(
        '--create-droplet',
        action='store_true',
        help='Create droplet automatically')
    group.add_argument(
        '--droplet-id',
        metavar='ID',
        type=int,
        help='ID of existing Greenhost VPS to use')

    droplet_creation_group = create_parser.add_argument_group(
        'droplet creation',
        'When using --create-droplet, you need to provide:')
    droplet_creation_group.add_argument(
        '--hostname',
        type=str,
        help='hostname of the new machine. If not provided, a hostname is generated')

    droplet_creation_group.add_argument(
        '--ssh-key-id',
        metavar='ID',
        type=int,
        help='ID of ssh key to deploy with (default: 411)')

    droplet_creation_group.add_argument(
        '--create-domain-records',
        action='store_true',
        help=('Creates DNS entries at Greenhost pointing the subdomain and '
              'domain to the cluster.'))

    droplet_creation_group.add_argument(
        '--subdomain',
        type=str,
        metavar="SUBDOMAIN",
        default="",
        help=('Use a custom subdomain for the generated domain records. '
              'Defaults to empty string (no subdomain)'))

    install_parser = subparsers.add_parser(
        'install',
        help=("Use this to run the ansible playbook that sets up your VPS to run "
              "OpenAppStack. The ansible-playbook process will run in the "
              "'{ansible_path}' directory, so do not use relative paths with these "
              "arguments.").format(ansible_path=ansible.ANSIBLE_PATH))
    install_parser.set_defaults(func=install)

    parser.add_argument(
        '--terminate',
        action='store_true',
        help=('Shutdown and delete droplet identified by VPS identifier after '
              'finishing'))

    parser.add_argument(
        '-v',
        '--verbose',
        action='store_true',
        help='Be more verbose')

    # Ansible related
    install_parser.add_argument(
        '--ansible-param',
        metavar=['PARAM[=VALUE]'],
        action='append',
        nargs=1,
        help=('forward ansible parameters to the ansible-playbook call '
              '(two dashes are prepended to PARAM)'))

    test_parser = subparsers.add_parser(
        'test',
        help=("Write test configuration and run tests on your cluster"))

    test_parser.set_defaults(func=test)

    test_parser.add_argument(
        '--behave',
        action='store_true',
        help='Runs "behave" tests. NOTE: This overrides behave.ini!')
    test_parser.add_argument(
        '--behave-rerun-failing',
        action='store_true',
        help=('Run behave with @rerun_failing.features'))
    test_parser.add_argument(
        '--behave-tags',
        nargs='+',
        default=[],
        help=('Only run behave tests with these tags'))
    test_parser.add_argument(
        '--behave-headless',
        action='store_true',
        help=('Run behave in headless mode'))

    info_parser = subparsers.add_parser(
        'info',
        help=("Show information about a cluster"))

    info_parser.set_defaults(func=info)

    args = parser.parse_args()
    loglevel = logging.DEBUG if args.verbose else logging.INFO
    init_logging(log, loglevel)

    # Setup logging for cosmos module
    log_cosmos = logging.getLogger('cosmos')
    init_logging(log_cosmos, loglevel)

    log.debug("Parsed arguments: %s", str(args))

    clus = cluster.Cluster(args.cluster_name)

    if hasattr(args, 'func'):
        args.func(clus, args)

    if args.terminate:
        # In case none of the subparser's functions have been called, load data
        clus.load_data()
        cosmos.terminate_droplets_by_name(clus.hostname)

def info(clus, _args):
    """Shows cluster information and then exits"""
    clus.load_data()
    clus.print_info()


def create(clus, args):
    """Parses arguments for the "create" subcommand"""
    clus = cluster.Cluster(args.cluster_name)

    if args.create_droplet:
        if not args.ssh_key_id:
            log.error("--ssh-key-id required when using --create-droplet")
            sys.exit(1)
        if not args.hostname:
            log.error("--hostname required when using --create-droplet")
            sys.exit(1)

    if args.subdomain:
        domain = "{subdomain}.{domain}".format(
            subdomain=args.subdomain, domain=args.domain)
    else:
        domain = args.domain
    clus.domain = domain
    if args.create_droplet:
        clus.create_droplet(ssh_key_id=args.ssh_key_id, hostname=args.hostname)
        if args.verbose:
            cosmos.list_droplets()
        # Wait for ssh
        cosmos.wait_for_ssh(clus.ip_address)
    elif args.droplet_id:
        clus.set_info_by_droplet_id(args.droplet_id)

    # Write inventory.yml and settings.yml files
    clus.write_cluster_files()


    if args.create_domain_records:
        create_domain_records(args.domain, args.subdomain, clus.ip_address)
        if args.verbose:
            cosmos.list_domain_records('openappstack.net')


def install(clus, args):
    """Parses arguments for the "install" subcommand"""
    clus.load_data()
    ansible.run_ansible(
        clus,
        os.path.join(ansible.ANSIBLE_PATH, 'bootstrap.yml'),
        args.ansible_param)


def test(clus, args):
    """Runs behave or testinfra test. Overwrites behave_path/behave.ini!"""
    if args.behave:
        behave_path = os.path.join(os.path.dirname(__file__), '..', 'test',
                                   'behave')
        # Run from the behave directory so behave automatically loads all the
        # necessary files
        os.chdir(behave_path)
        clus.load_data()
        clus.write_behave_config(os.path.join(behave_path, 'behave.ini'))
        command = []
        if args.behave_rerun_failing:
            command.append("@rerun_failing.features")
        log.info(args.behave_tags)
        if args.behave_headless:
            command.append("-D headless=True")
        for tag in args.behave_tags:
            log.info(command)
            command.append("-t {tag}".format(tag=tag))
        log.info("Running behave command %s", command)
        behave_main(command)

def create_domain_records(domain, subdomain, droplet_ip):
    """
    Creates domain records at Greenhost in the way OpenAppStack expects it
    """
    # Create domain records
    domain_record = cosmos.create_domain_record(
        domain=domain, name=subdomain,
        data=droplet_ip, record_type='A', update=True)
    log.info("Domain record: %s", domain_record)

    domain_record = cosmos.create_domain_record(
        domain=domain, name='*.' + subdomain,
        data=subdomain, record_type='CNAME', update=True)
    log.info("Domain record: %s", domain_record)




def init_logging(logger, loglevel):
    """
    Configure logging.

    - debug and info go to stdout
    - warning and above go to stderr
    """
    logger.setLevel(loglevel)
    stdout = logging.StreamHandler(sys.stdout)
    stdout.setLevel(loglevel)
    stdout.addFilter(lambda record: record.levelno <= logging.INFO)
    logger.info("Set log level to %d", loglevel)

    stderr = logging.StreamHandler()
    stderr.setLevel(logging.WARNING)

    logger.addHandler(stdout)
    logger.addHandler(stderr)


if __name__ == "__main__":
    # Setup logging for this script
    log = logging.getLogger("openappstack")  # pylint: disable=invalid-name
    main()
