"""
Module responsible for running the Ansible part of the OpenAppStack setup.
"""
import logging
import os
import shlex
import subprocess
import yaml

log = logging.getLogger(__name__)  # pylint: disable=invalid-name

ANSIBLE_INVENTORY = './clusters/{cluster_name}/inventory.yml'
ANSIBLE_PATH = os.path.join(os.path.dirname(__file__),
                            '..', "ansible")

def run_ansible(clus, playbook, ansible_params=None):
    """
    Call `ansible-playbook` in a subprocess to run the specified playbook. Runs
    in the package's ansible directory.

    Parameters:
    - playbook: path to the playbook to run.
    - ansible_params: Optionally provide a list of lists with ansible params.
      Each inner list may only contain one element. Can be directly forwarded
      from argparse.

      Example: ansible_params = [[become-user=root], [verbose]]
    """
    # playbook path here is relative to private_data_dir/project, see
    # https://ansible-runner.readthedocs.io/en/latest/intro.html#inputdir
    ansible_param_string = ""
    if ansible_params:
        for param in ansible_params:
            if len(param) > 1:
                log.warning('More than 1 parameter. Ignoring the rest! Use '
                            '--ansible-param several times to supply '
                            'more than 1 parameter')
            param = param[0]
            ansible_param_string += ' --' + param

    ansible_playbook_cmd = \
        ('ansible-playbook {ansible_param_string} -i {inventory} '
         '-e @{settings} {playbook}').format(
             ansible_param_string=ansible_param_string,
             inventory=clus.inventory_file,
             settings=clus.settings_file,
             playbook=playbook
         )

    log.info('Running "%s" in ansible directory "%s"',
             ansible_playbook_cmd,
             ANSIBLE_PATH)

    process = subprocess.Popen(
        shlex.split(ansible_playbook_cmd),
        cwd=ANSIBLE_PATH)

    returncode = process.wait()

    if returncode > 0:
        raise RuntimeError('Playbook failed with rc %s.'
                           % returncode)

def create_inventory(cluster):
    """
    Creates inventory for the ansible playbook. Needs the droplet's hostname
    for identification and the IP for connecting with Ansible
    """
    # Create inventory
    with open(os.path.join(ANSIBLE_PATH, "inventory.yml.example"),
              'r') as stream:
        inventory = yaml.safe_load(stream)

    inventory['all']['hosts'][cluster.hostname] = \
        inventory['all']['hosts']['oas-dev']
    del inventory['all']['hosts']['oas-dev']

    inventory['all']['hosts'][cluster.hostname]['ansible_host'] = \
        cluster.ip_address
    # Moved to ansible.cfg (how can we re-enable host key checking?)
    # inventory['all']['hosts'][cluster.hostname]['ansible_ssh_extra_args'] = \
    #     '-o UserKnownHostsFile=/dev/null -o StrictHostKeyChecking=no'
    inventory['all']['children']['master']['hosts'] = cluster.hostname
    inventory['all']['children']['worker']['hosts'] = cluster.hostname

    with open(cluster.inventory_file, 'w') as stream:
        yaml.safe_dump(inventory, stream, default_flow_style=False)
    log.debug(yaml.safe_dump(inventory, default_flow_style=False))
    return inventory
