"""Contains code for managing the files related to an OpenAppStack cluster"""

import configparser
import logging
import os
import random
import string
import yaml
from openappstack import ansible, cosmos

CLUSTER_PATH = os.path.join(os.getcwd(), "clusters")

log = logging.getLogger(__name__)  # pylint: disable=invalid-name

class Cluster:
    """helper class for cluster-related paths, files, etc."""


    def __init__(self, cluster_name, load_data=False):
        self.name = cluster_name
        self.cluster_dir = os.path.join(CLUSTER_PATH, cluster_name)
        self.ip_address = None
        self.hostname = None
        self.domain = None

        # Load data from inventory.yml and settings.yml
        if load_data:
            self.load_data()

    def load_data(self):
        """Loads cluster data from inventory.yml and settings.yml files"""
        # Work with the master node from the inventory

        with open(self.settings_file, 'r') as stream:
            settings = yaml.safe_load(stream)
            self.ip_address = settings['ip_address']
            self.domain = settings['domain']

        log.debug("""Read data from settings.yml:
            ip address: %s
            domain: %s""", self.ip_address, self.domain)

        with open(self.inventory_file, 'r') as stream:
            inventory = yaml.safe_load(stream)
        self.hostname = inventory['all']['children']['master']['hosts']

        log.debug("Read data from inventory.yml:\n\thostname: %s", self.hostname)


    def create_droplet(self, ssh_key_id=0, hostname=None):
        """Uses the Cosmos API to create a droplet with OAS default spec"""
        if hostname is None:
            # Use random generated ID in case we're not running in
            # gitlab CI and there's no CI_PIPELINE_ID env var
            hostname = ''.join(
                random.choice(string.ascii_lowercase + string.digits)
                for _ in range(10))
            log.info('Generated hostname %s', hostname)
        droplet = cosmos.create_droplet(
            name=hostname,
            ssh_key_id=ssh_key_id,
            region='ams1',
            size=4096,
            disk=25,
            # image: 19 = Debian buster-x64
            image=19)
        droplet_id = droplet['droplet']['id']
        log.info('Created droplet id: %s', droplet_id)
        cosmos.wait_for_state(droplet_id, 'running')
        self.set_info_by_droplet_id(droplet_id)

    def set_info_by_droplet_id(self, droplet_id):
        """Sets info about the cluster based on the Greenhost VPS id"""
        droplet = cosmos.get_droplet(droplet_id)
        self.ip_address = droplet['networks']['v4'][0]['ip_address']
        self.hostname = droplet['name']

    def write_cluster_files(self):
        """Creates an inventory.yml and settings.yml file for the cluster"""
        self.make_cluster_directory()
        ansible.create_inventory(self)

        # Create settings
        with open(os.path.join(ansible.ANSIBLE_PATH, "group_vars",
                               "all", "settings.yml.example"),
                  'r') as stream:
            settings = yaml.safe_load(stream)

        settings['ip_address'] = self.ip_address
        settings['domain'] = self.domain
        settings['admin_email'] = "admin@{0}".format(self.domain)
        settings['acme_staging'] = True
        settings['cluster_dir'] = self.cluster_dir

        with open(self.settings_file, 'w') as stream:
            yaml.safe_dump(settings, stream, default_flow_style=False)

        log.debug(yaml.safe_dump(settings, default_flow_style=False))

    def make_cluster_directory(self):
        """Make sure the cluster's file directory exists"""
        if not os.path.exists(self.cluster_dir):
            os.makedirs(self.cluster_dir)

    @property
    def inventory_file(self):
        """Path to the ansible inventory.yml for this cluster"""
        return os.path.join(self.cluster_dir, "inventory.yml")

    @property
    def settings_file(self):
        """Path to the ansible settings.yml for this cluster"""
        return os.path.join(self.cluster_dir, "settings.yml")

    @property
    def behave_file(self):
        """Path to 'behave.ini' which is used for acceptance tests"""
        return os.path.join(self.cluster_dir, "behave.ini")

    @property
    def secret_dir(self):
        """Path where all the passwords for cluster admins are saved"""
        return os.path.join(self.cluster_dir, 'secrets')

    def write_behave_config(self, config_path):
        """Write behave config file for the cluster."""
        secret_directory = self.secret_dir
        with open(os.path.join(
                secret_directory, 'nextcloud_admin_password'), 'r') as stream:
            nextcloud_admin_password = yaml.safe_load(stream)

        with open(os.path.join(
                secret_directory, 'grafana_admin_password'), 'r') as stream:
            grafana_admin_password = yaml.safe_load(stream)

        behave_config = configparser.ConfigParser()
        behave_config['behave'] = {}
        behave_config['behave']['format'] = 'rerun'
        behave_config['behave']['outfiles'] = 'rerun_failing.features'
        behave_config['behave']['show_skipped'] = 'false'

        behave_config['behave.userdata'] = {}

        behave_config['behave.userdata']['nextcloud.url'] = \
            'https://files.{}'.format(self.domain)
        behave_config['behave.userdata']['nextcloud.username'] = 'admin'
        behave_config['behave.userdata']['nextcloud.password'] = \
            nextcloud_admin_password

        behave_config['behave.userdata']['grafana.url'] = \
            'https://grafana.{}'.format(self.domain)
        behave_config['behave.userdata']['grafana.username'] = 'admin'
        behave_config['behave.userdata']['grafana.password'] = \
            grafana_admin_password

        with open(config_path, 'w') as config_file:
            behave_config.write(config_file)

    def print_info(self):
        """Writes information about the cluster. Useful for debugging"""
        print(("""Cluster "{name}":
  - IP address: {ip_address}
  - Hostname: {hostname}
  - Domain: {domain}

Configuration:
  - Inventory file: {inventory_file}
  - Settings file: {settings_file}""").format(
            name=self.name,
            ip_address=self.ip_address,
            hostname=self.hostname,
            domain=self.domain,
            inventory_file=self.inventory_file,
            settings_file=self.settings_file))
